TEMPLATES = {
    "record": "templates/invenio_record.py.jinja2",
    "record-metadata": "templates/invenio_record_metadata.py.jinja2",
    "record-permissions": "templates/invenio_record_permissions.py.jinja2",
    "record-search-options": "templates/invenio_record_search_options.py.jinja2",
    "record-service-config": "templates/invenio_record_service_config.py.jinja2",
    "record-service": "templates/invenio_record_service.py.jinja2",
    "record-schema": "templates/invenio_record_schema.py.jinja2",
    "object-schema": "templates/invenio_record_object_schema.py.jinja2",
    "record-dumper": "templates/invenio_record_dumper.py.jinja2",
    "record-resource-config": "templates/invenio_record_resource_config.py.jinja2",
    "record-resource": "templates/invenio_record_resource.py.jinja2",
    "views": "templates/invenio_views.py.jinja2",
    "invenio_version": "templates/invenio_version.py.jinja2",
    "config": "templates/invenio_config.py.jinja2",
    "ext": "templates/invenio_ext.py.jinja2",
    "proxies": "templates/invenio_proxies.py.jinja2",
    "record-facets": "templates/invenio_record_facets.py.jinja2",
    # utils and included
    "imports": "templates/imports.py.jinja2",
    # scripts
    "script-import-sample-data": "templates/script_import_sample_data.py.jinja2",
    "script-import-sample-data-shell": "templates/script_import_sample_data.sh.jinja2",
}
